/**
  ******************************************************************************
  * @file    retarget.c
  *
  * @brief   Содержит реализацию функции инициализации UART2, а также
  *          переопределенные библиотечные функции, относящиеся к printf.
  *          Также стоить обратить внимание, что для того чтобы все работало,
  *          в компилятор передается ключ -fno-builtin, а линкеру --specs=nosys.specs
  *
  *          Компилятор: GCC ARM 4.9.3
  *          Среда: Qt Creator 3.4.2
  *
  * @author  НИИЭТ
  *             - Богдан Колбов (bkolbov), kolbov@niiet.ru
  * @date    18.11.2015
  *
  ******************************************************************************
  * @attention
  *
  * ДАННОЕ ПРОГРАММНОЕ ОБЕСПЕЧЕНИЕ ПРЕДОСТАВЛЯЕТСЯ «КАК ЕСТЬ», БЕЗ КАКИХ-ЛИБО
  * ГАРАНТИЙ, ЯВНО ВЫРАЖЕННЫХ ИЛИ ПОДРАЗУМЕВАЕМЫХ, ВКЛЮЧАЯ ГАРАНТИИ ТОВАРНОЙ
  * ПРИГОДНОСТИ, СООТВЕТСТВИЯ ПО ЕГО КОНКРЕТНОМУ НАЗНАЧЕНИЮ И ОТСУТСТВИЯ
  * НАРУШЕНИЙ, НО НЕ ОГРАНИЧИВАЯСЬ ИМИ. ДАННОЕ ПРОГРАММНОЕ ОБЕСПЕЧЕНИЕ
  * ПРЕДНАЗНАЧЕНО ДЛЯ ОЗНАКОМИТЕЛЬНЫХ ЦЕЛЕЙ И НАПРАВЛЕНО ТОЛЬКО НА
  * ПРЕДОСТАВЛЕНИЕ ДОПОЛНИТЕЛЬНОЙ ИНФОРМАЦИИ О ПРОДУКТЕ, С ЦЕЛЬЮ СОХРАНИТЬ ВРЕМЯ
  * ПОТРЕБИТЕЛЮ. НИ В КАКОМ СЛУЧАЕ АВТОРЫ ИЛИ ПРАВООБЛАДАТЕЛИ НЕ НЕСУТ
  * ОТВЕТСТВЕННОСТИ ПО КАКИМ-ЛИБО ИСКАМ, ЗА ПРЯМОЙ ИЛИ КОСВЕННЫЙ УЩЕРБ, ИЛИ
  * ПО ИНЫМ ТРЕБОВАНИЯМ, ВОЗНИКШИМ ИЗ-ЗА ИСПОЛЬЗОВАНИЯ ПРОГРАММНОГО ОБЕСПЕЧЕНИЯ
  * ИЛИ ИНЫХ ДЕЙСТВИЙ С ПРОГРАММНЫМ ОБЕСПЕЧЕНИЕМ.
  *
  * <h2><center>&copy; 2015 ОАО "НИИЭТ"</center></h2>
  ******************************************************************************
  */

#include "retarget.h"

#define IBRD_REG_VAL UARTCLK_VAL/(16*BAUDRATE_VAL)
#define FBRD_REG_VAL ((UARTCLK_VAL/(16.0*BAUDRATE_VAL)-IBRD_REG_VAL)*64+0.5)

extern void retarget_init()
{
    //UART2
    NT_COMMON_REG->GPIOPCTLF&=~(15<<20);                //F10, F11 func1
    NT_GPIOF->DATAOUT=(1<<10);
    NT_GPIOF->OUTENSET=(1<<10);                         //F10 output
    NT_GPIOF->ALTFUNCSET=(1<<10)|(1<<11);               //F10,F11 altfunc en
    NT_COMMON_REG->GPIODEN2|=(1<<(10+16))|(1<<(11+16)); //F10(TX), F11(RX) out en

    NT_COMMON_REG->UART_SPI_CLK_SEL&=~(3<<4);           //UART2 sysclk
    NT_COMMON_REG->UART_CLK|=1<<16;                     //UART2 clk en
    NT_COMMON_REG->PER_RST1|=1<<9;                      //uart2 rst off

    NT_UART2->IBRD=IBRD_REG_VAL;
    NT_UART2->FBRD=FBRD_REG_VAL;
    NT_UART2->LCR_H=3<<5;                               //8bit
    NT_UART2->CR=(1<<0)|(1<<8)|(1<<9);                  //uart en, tx en, rx en

    setvbuf(stdout, NULL, _IONBF, 0);
}

int _write (int fd, char *ptr, int len)
{
    /* Write "len" of char from "ptr" to file id "fd"
     * Return number of char written.
     * Need implementing with UART here. */
    int i = 0;

    while (ptr[i] && (i < len))
    {
        while(NT_UART2->FR & 0x8);
        NT_UART2->DR = ptr[i];
        /*if (ptr[i] == '\n')
        {
            NT_UART2->DR = '\r';
        }*/
        i++;
    }

    return len;
}

void _ttywrch(int ch)
{
  /* Write one char "ch" to the default console
   * Need implementing with UART here. */
    while(NT_UART2->FR & 0x8);
    NT_UART2->DR = ch;
}

int _read(int file, char *ptr, int len)
{
    return 0;
}
